// CkCrypt2.h: interface for the CkCrypt2 class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _CKCRYPT2_H
#define _CKCRYPT2_H

#pragma once

class CkCSP;
class CkCert;
class CkCertStore;
class CkString;
class CkByteData;
#include "CkObject.h"

// CLASS: CkCrypt2
class CkCrypt2  : public CkObject
{
    private:
	void *m_impl;
	bool m_utf8;	// If true, all input "const char *" parameters are utf-8, otherwise they are ANSI strings.

	// Don't allow assignment or copying these objects.
	CkCrypt2(const CkCrypt2 &);
	CkCrypt2 &operator=(const CkCrypt2 &);

    public:
	void *getImpl(void);

	CkCrypt2(void *impl);

	CkCrypt2();
	virtual ~CkCrypt2();

	// BEGIN PUBLIC INTERFACE
	bool get_Utf8(void) const;
	void put_Utf8(bool b);


	bool SetCSP(CkCSP *csp);
	CkCert *GetLastCert(void);
	void SetEncryptCert(CkCert *cert);
	void SetSigningCert(CkCert *cert);
	void SetVerifyCert(CkCert *cert);

	bool CkEncryptFile(const char *inFile, const char *outFile);
	bool CkDecryptFile(const char *inFile, const char *outFile);

	bool VerifyStringENC(const char *str, const char *encodedSig);
	bool VerifyString(const char *str, const CkByteData &sigData);
	bool VerifyBytesENC(const CkByteData &bData, const char *encodedSig);
	bool VerifyBytes(const CkByteData &bData1, const CkByteData &sigData);

	bool SignStringENC(const char *str, CkString &out);
	bool SignBytesENC(const CkByteData &bData, CkString &out);
	bool SignString(const char *str, CkByteData &out);
	bool SignBytes(const CkByteData &bData, CkByteData &out);

	bool InflateStringENC(const char *str, CkString &out);
	bool InflateString(const CkByteData &bData, CkString &out);
	bool InflateBytesENC(const char *str, CkByteData &out);
	bool InflateBytes(const CkByteData &bData, CkByteData &out);

	bool CompressStringENC(const char *str, CkString &out);
	bool CompressBytesENC(const CkByteData &bData, CkString &out);
	bool CompressString(const char *str, CkByteData &out);
	bool CompressBytes(const CkByteData &bData, CkByteData &out);

	bool get_CompressionAlgorithm(CkString &out);
	void put_CompressionAlgorithm(const char *alg);

	bool DecryptStringENC(const char *str, CkString &out);
	bool DecryptString(const CkByteData &bData, CkString &out);
	bool DecryptBytesENC(const char *str, CkByteData &out);
	bool DecryptBytes(const CkByteData &bData, CkByteData &out);

	bool EncryptStringENC(const char *str, CkString &out);
	bool EncryptBytesENC(const CkByteData &bData, CkString &out);
	bool EncryptString(const char *str, CkByteData &out);
	bool EncryptBytes(const CkByteData &bData, CkByteData &out);

	bool HashBytesENC(const CkByteData &bData, CkString &out);
	bool HashStringENC(const char *str, CkString &out);
	bool HashBytes(const CkByteData &bData, CkByteData &out);
	bool HashString(const char *str, CkByteData &out);

	void get_EncodingMode(CkString &out);
	void put_EncodingMode(const char *str);
	void get_CryptAlgorithm(CkString &out);
	void put_CryptAlgorithm(const char *str);
	void get_HashAlgorithm(CkString &out);
	void put_HashAlgorithm(const char *str);
	void get_Charset(CkString &out);
	void put_Charset(const char *str);

	void GenerateSecretKey(const char *password, CkByteData &out);

	void get_CipherMode(CkString &out);
	void put_CipherMode(const char *newVal);
	long get_PaddingScheme(void);
	void put_PaddingScheme(long newVal);
	long get_KeyLength(void);
	void put_KeyLength(long newVal);
	void get_IV(CkByteData &bData);
	void put_IV(const CkByteData &bData);
	void get_SecretKey(CkByteData &bData);
	void put_SecretKey(const CkByteData &bData);
	void get_Version(CkString &out);

	bool IsUnlocked(void);
	bool UnlockComponent(const char *unlockCode);


	// Error log retrieval and saving (these methods are common to all Chilkat VC++ classes.)
	bool SaveLastError(const char *filename);
        void LastErrorXml(CkString &str);
        void LastErrorHtml(CkString &str);
        void LastErrorText(CkString &str);

	// END PUBLIC INTERFACE

};


#endif


